from .cache import CachedGetter
import requests


def _request_metadata(url: str, timeout: tuple[int, int]) -> dict:
    response = requests.get(url, timeout=timeout)
    return response.json()


class OidcEndpoints:
    _metadata_response: CachedGetter
    token: str
    introspection: str
    userinfo: str
    end_session: str
    jwks_uri: str

    def __init__(self, server_metadata_url: str, request_timeout: tuple[int, int]):
        self._metadata_response = CachedGetter(
            lambda: _request_metadata(server_metadata_url, request_timeout), 60 * 60
        )

    def update(self):
        self._metadata_response.update()

    def certs(self) -> dict:
        return self._metadata_response.get()["jwks_uri"]

    def issuer(self) -> str:
        return self._metadata_response.get()["issuer"]

    def set_from_well_known(self, well_known: any):
        self.token = well_known["token_endpoint"]
        self.introspection = well_known["introspection_endpoint"]
        self.userinfo = well_known["userinfo_endpoint"]
        self.end_session = well_known["end_session_endpoint"]
        self.jwks_uri = well_known["jwks_uri"]



from typing import Annotated
from fastapi import FastAPI, Request, Depends, HTTPException, Security
from fastapi.security import SecurityScopes
from ..core import Auth
from ..token import Token
from ..user import User

_SECURITY_SCHEME_NAME = "JWT"


def _security_scheme_docs(scheme_name: str) -> dict:
    return {
        scheme_name: {
            "type": "http",
            "scheme": "bearer",
            "bearerFormat": "JWT",
            "description": "JWT access token in the Authorization bearer format",
        }
    }


class FastAPIAuth:
    _auth: Auth

    def __init__(self, auth: Auth):
        self._auth = auth

    def add_docs(self, app: FastAPI, is_globally_protected: bool = True):
        """
        Adds the authentication scheme to the `app` openapi documentation.
        """
        original_schema = app.openapi()
        schema_set = False

        def custom_openapi():
            nonlocal schema_set
            if schema_set:
                return app.openapi_schema
            schema_set = True

            nonlocal original_schema
            schema = original_schema
            schema["security"] = (
                [{_SECURITY_SCHEME_NAME: []}] if is_globally_protected else []
            )
            if "components" not in schema:
                schema["components"] = {}
            schema["components"] |= {
                "securitySchemes": _security_scheme_docs(_SECURITY_SCHEME_NAME)
            }

            app.openapi_schema = schema
            return app.openapi_schema

        app.openapi = custom_openapi

    def scope_docs(self, scopes: list[str], is_resource_namespaced: bool = True):
        if is_resource_namespaced:
            scopes = [f"{self._auth.config.client_id}:{scope}" for scope in scopes]

        return {"security": [{_SECURITY_SCHEME_NAME: scopes}]}

    def unvalidated_token(self):
        """
        Dependency for the unvalidated token.
        Generally, you should use the validated token instead with the `FastAPIAuth.token` dependency instead.
        """
        # Note that it is named "unvalidated" instead of "invalidated" because
        # "invalid" means that it has been valid before, and became invalid.
        # "unvalid" instead means that it hasn't been checked whether it is valid.

        def dependency(request: Request) -> str:
            auth_header = request.headers.get("Authorization")
            if auth_header is None:
                raise HTTPException(status_code=401)
            return self._auth.get_token(auth_header)

        return dependency

    def token(self):
        def decorator(
            token: Annotated[str, Depends(self.unvalidated_token())],
        ) -> Token:
            return self._auth.validate_token(token)

        return decorator

    def user(self):
        def decorator(token: Annotated[Token, Depends(self.token())]) -> User:
            return token.user

        return decorator

    def _scopes(self, is_resource_namespaced: bool = True):
        def decorator(
            security_scopes: SecurityScopes,
            token: Annotated[Token, Depends(self.token())],
        ):
            if not token.has_scopes(
                security_scopes.scopes, is_resource_namespaced=is_resource_namespaced
            ):
                raise HTTPException(status_code=403)

        return decorator

    def require_scopes(self, scopes: list[str], is_resource_namespaced: bool = True):
        return Security(
            self._scopes(is_resource_namespaced=is_resource_namespaced), scopes=scopes
        )



from .core import Auth
from .config import AuthConfig
from .token import Token
from .user import User
from .fastapi.auth import FastAPIAuth

__all__ = ["Auth", "FastAPIAuth", "AuthConfig", "Token", "User"]



from dataclasses import dataclass


@dataclass
class AuthConfig:
    """Configuration for authentication and authorization.

    Most optional configuration options should be left at default, except `client_secret` if you need to validate a token remotely.

    Attributes:
        client_id: The client ID of this service.
            This is used to namespace the role checks, for example, when you use `Auth.require_role("my-role")`, then it will check for "my-role" under the `resource`.
            This is also used to authorize any calls to the user provider if needed, for example, when you validate a token remotely.
        client_secret: The secret for the client with ID equal to `client_id`.
            This is used to authorize any calls to the user provider if needed, for example, when you validate a token remotely.
        well_known_endpoint: The URL endpoint for OpenID configuration discovery.
            Used to fetch authentication provider metadata.
        request_timeout: Connection and read timeout in seconds as a tuple.
            First value is connection timeout, second is read timeout.
    """

    client_id: str
    client_secret: str = None
    well_known_endpoint: str = (
        "http://user.psp.svc.cluster.local/.well-known/openid-configuration/internal"
    )
    request_timeout: tuple[int, int] = (1, 5)



from joserfc import jwt
from .user import User


class Token:
    _token: jwt.Token
    _resource: str

    def __init__(self, _token: jwt.Token, _resource: str):
        self._token = _token
        self._resource = _resource

    @property
    def claims(self) -> dict:
        return self._token.claims

    @property
    def issuer(self) -> str:
        return self.claims.get("iss")

    @property
    def user(self) -> User:
        return User(self.claims, self._resource)

    @property
    def expires_at(self) -> int:
        return self.claims["exp"]

    @property
    def issued_at(self) -> int:
        return self.claims["iat"]

    @property
    def token_id(self) -> str:
        """
        Returns the ID of this token.
        """
        # Note that this function is explicitly called "token_id", instead of just "id",
        # to avoid users accidentally using token.id instead of token.user.id.
        return self.claims["jti"]

    @property
    def authorized_party(self) -> str:
        """
        Returns which client requested the token.
        """
        return self.claims["azp"]

    @property
    def audience(self) -> list[str]:
        """
        Returns which clients are authorized to use the token.
        """
        return self.claims["aud"]

    @property
    def allowed_origins(self) -> list[str]:
        return self.claims["allowed_origins"]

    @property
    def scopes(self) -> list[str]:
        scopes = self.claims["scope"]
        if scopes == "":
            return []
        else:
            return self.claims["scope"].split(" ")

    @property
    def session_id(self) -> str:
        return self.claims["sid"]

    @property
    def authentication_class(self) -> str:
        """
        Returns the authentication class used.

        Values have the following meanings:
        "0": Anonymous authentication
        "1": Basic authentication (username/password)
        "2": Multi-factor authentication
        There may be other custom values as well.
        """
        return self.claims["acr"]

    def has_scopes(
        self, scopes: list[str], is_resource_namespaced: bool = True
    ) -> bool:
        token_scopes = self.scopes
        if is_resource_namespaced:
            scopes = map(lambda scope: f"{self._resource}:{scope}", scopes)
        return all(scope in token_scopes for scope in scopes)



from starlette.requests import Request
from typing import Annotated
from fastapi import Depends, status
from psp_auth import Token, User
from psp_auth.testing import MockToken
from psp_auth.fastapi.auth import _SECURITY_SCHEME_NAME as SECURITY_SCHEME_NAME


def test_unvalid_token(client, app, fauth, mauth):
    token_value = "hellothere"

    @app.get("/")
    async def get_token(token: Annotated[str, Depends(fauth.unvalidated_token())]):
        assert token is not None
        assert token == token_value
        return "ok"

    response = client.get("/", headers=mauth.auth_header(token_value))
    assert response.status_code == status.HTTP_200_OK


def test_token(client, app, fauth, mauth):
    token_value = mauth.issue_token(MockToken())

    @app.get("/")
    async def get_token(token: Annotated[Token, Depends(fauth.token())]):
        assert token is not None
        return "ok"

    response = client.get("/", headers=mauth.auth_header(token_value))
    assert response.status_code == status.HTTP_200_OK


def test_user(client, app, fauth, mauth):
    token_value = mauth.issue_token(MockToken())

    @app.get("/")
    async def get_token(user: Annotated[User, Depends(fauth.user())]):
        assert user is not None
        return "ok"

    response = client.get("/", headers=mauth.auth_header(token_value))
    assert response.status_code == status.HTTP_200_OK


def test_has_required_non_namespaced_scopes(client, app, fauth, mauth):
    scopes = ["testscope1", "scopetest2"]
    token = mauth.issue_token(MockToken(scopes=scopes), is_resource_namespaced=False)

    @app.get(
        "/",
        dependencies=[fauth.require_scopes(scopes, is_resource_namespaced=False)],
    )
    async def route(request: Request):
        return "ok"

    response = client.get("/", headers=mauth.auth_header(token))
    assert response.status_code == status.HTTP_200_OK


def test_has_required_namespaced_scopes(client, app, auth_config, fauth, mauth):
    scopes = [
        "testscope1",
        "scopetest2",
    ]

    token = mauth.issue_token(MockToken(scopes=scopes), is_resource_namespaced=True)

    @app.get(
        "/",
        dependencies=[fauth.require_scopes(scopes, is_resource_namespaced=True)],
    )
    async def route(request: Request):
        return "ok"

    response = client.get("/", headers=mauth.auth_header(token))
    assert response.status_code == status.HTTP_200_OK


def test_has_no_required_scopes(client, app, fauth, mauth):
    scopes = ["testscope1", "scopetest2"]
    token = MockToken()

    @app.get("/", dependencies=[fauth.require_scopes(scopes)])
    async def route(request: Request):
        return "ok"

    response = client.get("/", headers=mauth.auth_header(mauth.issue_token(token)))
    assert response.status_code == status.HTTP_403_FORBIDDEN


def test_has_namespaced_scope_but_not_scope_not_namespaced(client, app, fauth, mauth):
    scopes = ["testscope1", "scopetest2"]
    token = mauth.issue_token(MockToken(scopes=scopes), is_resource_namespaced=True)

    @app.get(
        "/",
        dependencies=[fauth.require_scopes(scopes, is_resource_namespaced=False)],
    )
    async def route(request: Request):
        return "ok"

    response = client.get("/", headers=mauth.auth_header(token))
    assert response.status_code == status.HTTP_403_FORBIDDEN


def test_has_some_required_scopes(client, app, fauth, mauth):
    scopes = ["testscope1", "scopetest2"]
    token = MockToken(scopes=[scopes[0]])

    @app.get("/", dependencies=[fauth.require_scopes(scopes)])
    async def route(request: Request):
        return "ok"

    response = client.get("/", headers=mauth.auth_header(mauth.issue_token(token)))
    assert response.status_code == status.HTTP_403_FORBIDDEN


def test_app_docs_security_schemes(client, app, fauth, mauth):
    fauth.add_docs(app)

    security_scheme = app.openapi()["components"]["securitySchemes"][
        SECURITY_SCHEME_NAME
    ]
    assert security_scheme["type"] == "http"
    assert security_scheme["scheme"] == "bearer"
    assert security_scheme["bearerFormat"] == "JWT"


def test_app_docs_security_global(client, app, fauth, mauth):
    fauth.add_docs(app)
    assert app.openapi()["security"] == [{SECURITY_SCHEME_NAME: []}]


def test_app_docs_security_not_global(client, app, fauth, mauth):
    fauth.add_docs(app, is_globally_protected=False)
    assert app.openapi()["security"] == []


def test_scope_docs(client, app, fauth, mauth):
    scopes = ["testscope1", "scopetest2"]
    token = MockToken(scopes=scopes)

    @app.get(
        "/",
        dependencies=[fauth.require_scopes(scopes)],
        openapi_extra=fauth.scope_docs(scopes),
    )
    async def route(request: Request):
        return "ok"

    fauth.add_docs(app)

    response = client.get("/", headers=mauth.auth_header(mauth.issue_token(token)))
    assert response.status_code == status.HTTP_200_OK

    assert app.openapi()["paths"]["/"]["get"]["security"][0][SECURITY_SCHEME_NAME] == [
        mauth.resource_namespace_scope(scope) for scope in scopes
    ]



from typing import Annotated
from fastapi import Depends
from psp_auth import Token
from psp_auth.testing import MockToken


def test_token_info(client, app, fauth, mauth):
    mock_token = MockToken()
    mock_user = mock_token.user

    @app.get("/")
    async def route(token: Annotated[Token, Depends(fauth.token())]):
        assert token.issuer == mock_token.issuer
        assert token.expires_at == mock_token.expires_at
        assert token.issued_at == mock_token.issued_at
        assert token.token_id == mock_token.token_id
        assert token.authorized_party == mock_token.authorized_party
        assert token.audience == (mock_token.audience + [mauth._client_id])
        assert token.allowed_origins == mock_token.allowed_origins
        assert token.scopes == mock_token.scopes
        assert token.session_id == mock_token.session_id
        assert token.authentication_class == mock_token.authentication_class

        user = token.user
        assert user.id == mock_user.id
        assert user.given_name == mock_user.given_name
        assert user.family_name == mock_user.family_name
        assert user.full_name == mock_user.full_name
        assert user.username == mock_user.username
        assert user.email == mock_user.email
        assert user.is_email_verified == mock_user.is_email_verified
        assert user.principal_name == mock_user.principal_name

        return "ok"

    response = client.get("/", headers=mauth.auth_header(mauth.issue_token(mock_token)))
    assert response.status_code == 200



import pytest
import joserfc
from psp_auth.testing import MockToken


def test_wrong_audience(client, app, auth, mauth):
    audience = ["mytestaudience"]
    mock_token = MockToken(audience=audience)
    token = mauth.issue_token(mock_token, add_client_as_audience=False)
    with pytest.raises(joserfc.errors.InvalidClaimError, match="Invalid claim: 'aud'"):
        auth.validate_token(token)


def test_correct_audience(client, app, auth, mauth):
    mock_token = MockToken()
    token = mauth.issue_token(mock_token, add_client_as_audience=True)
    assert auth.validate_token(token)


def test_multiple_correct_audiences(client, app, auth, mauth):
    """
    Tests for multiple audiences, where one of them is the correct one.
    """
    audience = ["mytestaudience"]
    mock_token = MockToken(audience=audience)
    token = mauth.issue_token(mock_token, add_client_as_audience=True)
    assert auth.validate_token(token)



from psp_auth.core import Auth
from joserfc import jwt
from joserfc.jwk import RSAKey
import time
from dataclasses import dataclass, field

DEFAULT_ISSUER = "https://auth.testing.psp.com/realms/psp"


def _add_prefix_to_all(prefix: str, v: list[str]) -> list[str]:
    return [f"{prefix}{x}" for x in v]


@dataclass
class MockUser:
    id: str = "057144ec-9588-4cc8-a9d7-3b3a040080b5"
    given_name: str = "John"
    family_name: str = "Doe"
    username: str = "jandoener123"
    email: str = "john@doe.com"
    is_email_verified: bool = True
    principal_name: str = "john@doe.com"

    roles: list[str] = field(default_factory=list)

    """
    Resource roles should be in the form: resource -> list[roles].
    You can add global roles by defining roles for `resource == "global"`.
    """
    resource_roles: dict[str, list[str]] = field(default_factory=dict)

    @property
    def full_name(self) -> str:
        return f"{self.given_name} {self.family_name}"

    def _claims(self, default_resource: str) -> dict[str, any]:
        claims = {
            "sub": self.id,
            "upn": self.principal_name,
            "email": self.email,
            "email_verified": self.is_email_verified,
            "given_name": self.given_name,
            "family_name": self.family_name,
            "name": self.full_name,
            "preferred_username": self.username,
        }
        claims |= self._role_claims(default_resource)
        return claims

    def _role_claims(self, default_resource: str) -> dict[str, any]:
        def roles_dict(roles: list[str]) -> dict[str, list[str]]:
            return {"roles": roles}

        claims = {}

        global_roles = self.resource_roles.pop("global", None)
        if global_roles is not None:
            claims["realm_access"] = roles_dict(global_roles)

        resource_access = {}
        if self.roles:
            resource_access[default_resource] = roles_dict(self.roles)

        if self.resource_roles:
            for resource, roles in self.resource_roles.items():
                resource_access[resource] = roles_dict(roles)

        claims["resource_access"] = resource_access

        return claims


@dataclass
class MockToken:
    issuer: str = DEFAULT_ISSUER
    user: MockUser = field(default_factory=MockUser)
    scopes: list[str] = field(default_factory=list)
    issued_at: int = field(default_factory=lambda: int(time.time()))
    expires_at: int = field(
        default_factory=lambda: int(time.time()) + 3600
    )  # Expires in 1 hour
    token_id: str = "onrtrt:00b4dfb6-ad71-24de-9db5-dc5e9383c14f"
    authorized_party: str = "users"
    audience: list[str] = field(default_factory=list)
    session_id: str = "4a31869d-dc4a-4727-b28a-be5c92a16f4b"
    """
    The security class at which the user has authenticated themselves.

    Values have the following meanings:
    "0": Anonymous authentication
    "1": Basic authentication (username/password)
    "2": Multi-factor authentication
    There may be other custom values as well.
    """
    authentication_class: str = "2"
    allowed_origins: list[str] = field(default_factory=lambda: ["/*"])

    def _claims(
        self,
        default_resource: str,
        extra_audience: list[str] = [],
        prepend_resource_to_scopes: bool = True,
    ) -> dict:
        scopes = (
            _add_prefix_to_all(f"{default_resource}:", self.scopes)
            if prepend_resource_to_scopes
            else self.scopes
        )

        claims = {
            "iss": self.issuer,
            "exp": self.expires_at,
            "iat": self.issued_at,
            "jti": self.token_id,
            "azp": self.authorized_party,
            "typ": "Bearer",
            "aud": (self.audience if self.audience else []) + extra_audience,
            "scope": " ".join(scopes),
            "sid": self.session_id,
            "acr": self.authentication_class,
            "allowed_origins": self.allowed_origins,
        }
        claims |= self.user._claims(default_resource)
        return claims


def _generate_private_key() -> RSAKey:
    return RSAKey.generate_key(2048, auto_kid=True)


def _public_certs_from_key(key: RSAKey) -> dict:
    public_jwk = key.as_dict(private=False)
    return {"keys": [public_jwk]}


class MockAuth:
    def __init__(self, client_id: str, monkeypatch, issuer: str = DEFAULT_ISSUER):
        self._issuer = issuer
        self._client_id = client_id
        self._private_key = _generate_private_key()
        self._public_certs = _public_certs_from_key(self._private_key)
        self._mock_auth(monkeypatch)

    def _mock_auth(self, monkeypatch) -> None:
        public_certs = self._public_certs
        issuer = self._issuer

        def mock_token_certs(self):
            nonlocal public_certs
            return public_certs

        def mock_token_issuer(self):
            nonlocal issuer
            return issuer

        monkeypatch.setattr(Auth, "token_certs", mock_token_certs)
        monkeypatch.setattr(Auth, "token_issuer", mock_token_issuer)

    def auth_header(self, token: str) -> dict:
        return {"Authorization": self.auth_header_value(token)}

    def auth_header_value(self, token: str) -> str:
        return f"Bearer {token}"

    def issue_token(
        self,
        token: MockToken,
        add_client_as_audience: bool = True,
        is_resource_namespaced: bool = True,
    ):
        token = jwt.encode(
            header={"alg": "RS256", "kid": self._private_key.kid, "typ": "JWT"},
            claims=token._claims(
                self._client_id,
                extra_audience=[self._client_id] if add_client_as_audience else [],
                prepend_resource_to_scopes=is_resource_namespaced,
            ),
            key=self._private_key,
        )

        return token

    def resource_namespace_scope(self, scope: str) -> str:
        return f"{self._client_id}:{scope}"



class User:
    _claims: dict
    _resource: str

    def __init__(self, _claims: dict, _resource: str):
        self._claims = _claims
        self._resource = _resource

    @property
    def id(self) -> str:
        return self._claims["sub"]

    @property
    def principal_name(self) -> str:
        return self._claims["upn"]

    @property
    def email(self) -> str | None:
        return self._claims.get("email")

    @property
    def is_email_verified(self) -> bool | None:
        return self._claims.get("email_verified")

    @property
    def given_name(self) -> str | None:
        return self._claims.get("given_name")

    @property
    def family_name(self) -> str | None:
        return self._claims.get("family_name")

    @property
    def full_name(self) -> str | None:
        return self._claims.get("name")

    @property
    def username(self) -> str:
        """
        WARNING: Should _not_ be used as an identifier since it may change. Use `User.id` instead.
        """
        return self._claims["preferred_username"]

    def has_resource_role(self, resource: str, role: str) -> bool:
        """
        Args:
            resource: The resource. If `resource == "global"`, then it will check for global roles.
            roles: The roles.

        Returns:
            Whether the user has the `role` on the `resource`.
        """
        return self.has_any_resource_role(resource, [role])

    def has_any_resource_role(self, resource: str, roles: list[str]) -> bool:
        """
        Args:
            resource: The resource. If `resource == "global"`, then it will check for global roles.
            roles: The roles.

        Returns:
            Whether the user has one of the `roles` on the `resource`.
        """

        resource_roles = self._get_resource_roles(resource)
        if resource_roles is None:
            return False

        return any(role in resource_roles for role in roles)

    def has_all_resource_roles(self, resource: str, roles: list[str]) -> bool:
        """
        Args:
            resource: The resource. If `resource == "global"`, then it will check for global roles.
            roles: The roles.

        Returns:
            Whether the user has all the `roles` on the `resource`.
        """

        resource_roles = self._get_resource_roles(resource)
        if resource_roles is None:
            return False

        return all(role in resource_roles for role in roles)

    def _get_resource_roles(self, resource: str) -> dict | None:
        if resource == "global":
            return self._claims["realm_access"]["roles"]

        access = self._claims.get("resource_access")
        if access is None:
            return None

        resource_access = access.get(resource)
        if resource_access is None:
            return None

        resource_roles = resource_access["roles"]
        if resource_roles is None:
            return None

        return resource_roles

    def has_role(self, role: str) -> bool:
        """
        Returns:
            Whether the user has the given role.
        """
        return self.has_resource_role(self._resource, role)

    def has_any_role(self, roles: list[str]) -> bool:
        """
        Returns:
            Whether the user has any of the given roles
        """
        return self.has_any_resource_role(self._resource, roles)

    def has_all_roles(self, roles: list[str]) -> bool:
        """
        Returns:
            Whether the user has all the given roles
        """
        return self.has_all_resource_roles(self._resource, roles)



import time


class CachedGetter:
    """
    A class that caches the value of a getter for a certain amount of time.
    :param getter: A function that returns the value to be cached. If the function raises a DontCacheException, the cache will not be updated.
    :param time_until_cache: The time in seconds until the cache expires.
    """

    def __init__(self, getter, time_until_cache: int):
        self.getter = getter
        self.value = None
        self.time_until_cache = time_until_cache
        self.last_update = 0  # Last time the value was updated

    def update(self):
        try:
            self.value = self.getter()
            self.last_update = time.time()
        except DontCacheException:
            pass

    def get(self):
        if time.time() - self.last_update > self.time_until_cache:
            self.update()

        return self.value


class DontCacheException(Exception):
    pass



from fastapi import HTTPException
from joserfc import jwt
from joserfc.jwk import KeySet
from joserfc.jwt import JWTClaimsRegistry
import requests

from .config import AuthConfig
from .endpoints import OidcEndpoints
from .token import Token


class Auth:
    """
    Implements authentication and authorisation.
    """

    config: AuthConfig
    logger: any
    _endpoints: OidcEndpoints

    def __init__(self, config: AuthConfig):
        """
        Args:
            config: The auth configuration.
        """
        self.config = config
        self._endpoints = OidcEndpoints(
            self.config.well_known_endpoint, self.config.request_timeout
        )

    def _resource(self) -> str:
        return self.config.client_id

    def token_certs(self) -> dict:
        return requests.get(
            self._endpoints.certs(), timeout=self.config.request_timeout
        ).json()

    def token_issuer(self) -> str:
        return self._endpoints.issuer()

    def validate_token(self, token: str) -> Token:
        """
        Authorizes the token locally and returns it.
        """
        key_set = KeySet.import_key_set(self.token_certs())
        token = jwt.decode(token, key_set)
        claims_requests = JWTClaimsRegistry(
            iss={"essential": True, "value": self.token_issuer()},
            aud={"essential": True, "value": self._resource()},
        )
        claims_requests.validate(token.claims)
        return Token(token, self._resource())

    def get_token(self, auth_header: str) -> str:
        """
        Raises:
        - If `auth_header` has incorrect format, it will raise an HTTPException.
        """
        # Extract token from "Bearer <token>"
        parts = auth_header.split()

        if len(parts) != 2 or parts[0].lower() != "bearer":
            raise HTTPException(status_code=401, detail="Invalid Authorization header")

        return parts[1]

    async def validate_token_remotely(self, token: Token):
        """
        Authorizes the token remotely to verify that it has not been revoked.
        """
        raise NotImplementedError()
